@@ -1,9 +1,10 @@
 use std::{convert::TryFrom, marker::PhantomData};
 
 use crate::{
-    graph::csr::CsrLayout,
+    graph::csr::{CsrLayout, NodeValues},
     index::Idx,
     input::{edgelist::EdgeList, InputCapabilities, InputPath},
+    prelude::edgelist::{EdgeIterator, EdgeWithValueIterator},
     Error,
 };
 use std::path::Path as StdPath;
@@ -22,6 +23,15 @@ where
     _node: PhantomData<NI>,
 }
 
+pub struct FromEdgeListAndNodeValues<NI, NV, EV>
+where
+    NI: Idx,
+{
+    csr_layout: CsrLayout,
+    node_values: NodeValues<NV>,
+    edge_list: EdgeList<NI, EV>,
+}
+
 pub struct FromEdgesWithValues<NI, Edges, EV>
 where
     NI: Idx,
@@ -195,7 +205,7 @@ impl GraphBuilder<Uninitialized> {
     /// ```
     /// use graph::prelude::*;
     ///
-    /// let graph: DirectedCsrGraph<usize, f32> = GraphBuilder::new()
+    /// let graph: DirectedCsrGraph<usize, (), f32> = GraphBuilder::new()
     ///     .edges_with_values(vec![(0, 1, 0.1), (0, 2, 0.2), (1, 2, 0.3), (1, 3, 0.4), (2, 3, 0.5)])
     ///     .build();
     ///
@@ -248,7 +258,7 @@ impl GraphBuilder<Uninitialized> {
     /// ```
     /// use graph::prelude::*;
     ///
-    /// let g: UndirectedCsrGraph<usize, f32> = GraphBuilder::new()
+    /// let g: UndirectedCsrGraph<usize, (), f32> = GraphBuilder::new()
     ///     .gdl_str::<usize, _>("(a)-[{f: 0.42}]->(),(a)-[{f: 13.37}]->()")
     ///     .build()
     ///     .unwrap();
@@ -382,19 +392,32 @@ where
     NI: Idx,
     Edges: IntoIterator<Item = (NI, NI)>,
 {
+    pub fn node_values<NV, I>(
+        self,
+        node_values: I,
+    ) -> GraphBuilder<FromEdgeListAndNodeValues<NI, NV, ()>>
+    where
+        I: IntoIterator<Item = NV>,
+    {
+        let edge_list = EdgeList::from(EdgeIterator(self.state.edges));
+        let node_values = node_values.into_iter().collect::<NodeValues<NV>>();
+
+        GraphBuilder {
+            state: FromEdgeListAndNodeValues {
+                csr_layout: self.state.csr_layout,
+                node_values,
+                edge_list,
+            },
+        }
+    }
+
     /// Build the graph from the given vec of edges.
     pub fn build<Graph>(self) -> Graph
     where
         Graph: From<(EdgeList<NI, ()>, CsrLayout)>,
     {
         Graph::from((
-            EdgeList::new(
-                self.state
-                    .edges
-                    .into_iter()
-                    .map(|(s, t)| (s, t, ()))
-                    .collect(),
-            ),
+            EdgeList::from(EdgeIterator(self.state.edges)),
             self.state.csr_layout,
         ))
     }
@@ -406,6 +429,25 @@ where
     EV: Sync,
     Edges: IntoIterator<Item = (NI, NI, EV)>,
 {
+    pub fn node_values<NV, I>(
+        self,
+        node_values: I,
+    ) -> GraphBuilder<FromEdgeListAndNodeValues<NI, NV, EV>>
+    where
+        I: IntoIterator<Item = NV>,
+    {
+        let edge_list = EdgeList::from(EdgeWithValueIterator(self.state.edges));
+        let node_values = node_values.into_iter().collect::<NodeValues<NV>>();
+
+        GraphBuilder {
+            state: FromEdgeListAndNodeValues {
+                csr_layout: self.state.csr_layout,
+                node_values,
+                edge_list,
+            },
+        }
+    }
+
     /// Build the graph from the given vec of edges.
     pub fn build<Graph>(self) -> Graph
     where
@@ -418,6 +460,19 @@ where
     }
 }
 
+impl<NI: Idx, NV, EV> GraphBuilder<FromEdgeListAndNodeValues<NI, NV, EV>> {
+    pub fn build<Graph>(self) -> Graph
+    where
+        Graph: From<(NodeValues<NV>, EdgeList<NI, EV>, CsrLayout)>,
+    {
+        Graph::from((
+            self.state.node_values,
+            self.state.edge_list,
+            self.state.csr_layout,
+        ))
+    }
+}
+
 impl<NI> GraphBuilder<FromGdlString<NI>>
 where
     NI: Idx,
